#include<arpa/inet.h>
#include<sys/socket.h>
// #include <armadillo>
#include "boost/multi_array.hpp"
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/lu.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <math.h>
#include "UDPlib.h"


using namespace std;
// using namespace arma;
using namespace boost;
using namespace boost::numeric::ublas;

typedef boost::multi_array<short, 4> matrix4D;
typedef boost::numeric::ublas::matrix<int> matrix2Di;
typedef boost::numeric::ublas::matrix<double> matrix2Dd;
typedef matrix4D::index idx4D;
typedef matrix4D::index_range range4D;

#define PI 3.14159265
// #define MAX_SYNAPSES 50

// Defines related to pwm driver
 
#define NUMMOTORS               2
#define MOTOR0                  0
#define MOTOR1                  1
 
#define MAGIC_NUM 100
#define PWM_VERSION_READ        _IOR(MAGIC_NUM, 0, int *)
#define PWM_PERIOD_WRITE        _IOW(MAGIC_NUM, 1, int *)
#define PWM_ENABLE              _IOW(MAGIC_NUM, 2, int *)
#define PWM_MIN_H               _IOW(MAGIC_NUM, 3, int *)
#define PWM_MAX_H               _IOW(MAGIC_NUM, 4, int *)
#define PWM_MIN_V               _IOW(MAGIC_NUM, 5, int *)
#define PWM_MAX_V               _IOW(MAGIC_NUM, 6, int *)
 
/*
* Period and PWM limits are expressed in 100ns unit that is the clock period feeding .
* This means that:
*   PERIOD=200000 means 200000*100ns=20ms
*   PWMMIN=9000 means 0.9ms
*   PWMMAX=21000 means 2.1ms
*
*/
#define PERIOD                  200000
#define H_MIN_DUTY               17710
#define H_MAX_DUTY               21000
#define V_MIN_DUTY               15010
#define V_MAX_DUTY               18270
#define STEP_DUTY 		    10 //each step is 100ns*STEP_DUTY
#define SCALE_RANGE_MOTOR 	     1
 
#define MOTOR_ADDR   "/dev/pwm_dev" //""/dev/stdin" 

class dataset
{
  public:
    
      int bins,trials,channels,stim;
      string fileName;
      matrix4D data;
      
      
      dataset();
      dataset(int b,int t,int c,int s);
      dataset(int b,int t,int c,int s,string n);
      ~dataset();
      
      void changeSize(int b,int t,int c,int s);
};

class brainNode
{
  public:
    
    int output_neuron,output_synapse;
    float weight;
    
    brainNode();
    brainNode(int n,int s);
    brainNode(int n,int s,float w);
    ~brainNode();
    
};


class sensoryField
{
  public:
    
    int rangeX[2],rangeY[2],regions,resolution;
    double zero[2];
    matrix2Di map;
    
    sensoryField();
    sensoryField(int* w,int* h,double* z,int reg,int res);
    ~sensoryField();
    
    int whichRegion(double x, double y);
  
};

class forceField
{
  public:
    
    int rangeX[2],rangeY[2];
    double zero[2];
    matrix2Dd stiffness;
    std::vector<std::vector<double> > meanForce;
    
    forceField();
    forceField(int w[2], int h[2], double z[2],  double xxstiff,double yystiff,sensoryField SF);
    ~forceField();

    void computeForce(double x, double y, std::vector< double > *F);
};

class dynamicParam
{
  public:
    
    matrix2Dd mass,damping;
    int deltaTime,interTime,rangeX[2],rangeY[2];
    double minField[2],radius;
    
    dynamicParam();
    dynamicParam(double xxmass, double yymass, double xxdamp, double yydamp, int* w, int* h, double* z, double rad, int rT, int tT);
    ~dynamicParam();
  
};

class motorHandle
{
  public:
  
  int h_min_duty,h_max_duty,v_min_duty,v_max_duty;
  int h_max_step,v_max_step,rangeX[2],rangeY[2];
  double h_gain,v_gain;
  int curr_duty[2],step_duty;
  int mot_file;
  
  motorHandle();
  motorHandle(int* w,int* h);
  ~motorHandle();
  
  void pos2duty(std::vector< double > position);
};

void initMotor(motorHandle *MH);

void moveMotor(std::vector< double > position,motorHandle MH);

void stopMotor(motorHandle MH);

void calibrateMotor(motorHandle MH);

void importDatasetSize(int s, sockaddr_in si_me, sockaddr_in si_other,int* bins,int* trials,int* channels,int* stim);

void importDataset(int s, sockaddr_in si_me, sockaddr_in si_other, dataset* bmiData);

void importDataset2(int s, sockaddr_in si_me, sockaddr_in si_other, dataset* bmiDataTrain , dataset* bmiDataTest);

void StringToVector(string str, std::vector<short>* vec);

void StringTo4DMatrix(string str, dataset* bmiData, idx4D trial, idx4D channel, idx4D stim);

void pause( unsigned long ms );

int loadData(dataset* bmiData);

void decoder(forceField motorMap, std::vector< float > outputLayer,double gainFactor, std::vector< double >* F);

bool DynSys(dynamicParam DS, std::vector< double >* position, std::vector< double >* velocity, std::vector< double > Force, motorHandle MH, int s, sockaddr_in si_me, sockaddr_in si_other);

void inv(matrix<double> InputMatrix, matrix<double> &InverseMatrix);

void StringShortTo4DMatrix(string str, dataset* bmiData, idx4D trial, idx4D channel, idx4D stim);
