#include "BMIlib.h"
#include "boost/multi_array.hpp"
#include "boost/numeric/ublas/matrix.hpp"
#include <math.h>


//void LSMRunningFake(int curr_stim,std::vector< std::vector< float > >* outputLayer); //da eliminare

/////////////////////////
#define MN256_ADDR  "/dev/neuelab"
#define MN256_CLOCK_HZ 12500000 //(f=12.5 MHz -> dt=80ns)

#define MAGIC_NUM 100
#define NEUELAB_VERSION  	_IOR (MAGIC_NUM, 7, void *)
#define NEUELAB_GETTIMESTAMP    _IOR (MAGIC_NUM, 8, void *)
#define NEUELAB_GEN_REG		_IOWR(MAGIC_NUM, 6, void *)
#define NEUELAB_CLEARTIMESTAMP  _IOW (MAGIC_NUM, 9, void *)
#define NEUELAB_OUTNEUTHRESHOLD _IOW (MAGIC_NUM, 10, void *)
#define NEUELAB_SETLOOPBACK     _IOWR(MAGIC_NUM, 11, void *)

#define CTRL_REG   0x00
#define RXDATA_REG 0x08
#define RXTIME_REG 0x0C
#define TXDATA_REG 0x10
#define DMA_REG    0x14
#define RAWI_REG   0x18
#define IRQ_REG    0x1C
#define MASK_REG   0x20
#define BIAS_REG   0x24
#define STMP_REG   0x28
#define PRESC_REG  0x2C
#define OUTNEUTH_REG  0x30
#define ID_REG     0x5c
#define MASK_TX_AFULL 0x10
#define MASK_TX_FULL 0x20
#define MASK_RX_EMPTY 0x01
#define MASK_RX_FULL 0x04


//connectivity parameters
#define OFF_SINGLE_SPIKE 0
#define OFF_BRD_SPIKE 133120
#define OFF_SET_NONPLASTIC 133632
#define OFF_SET_PLASTIC_LATCH 788992
#define OFF_SET_PLASTIC_HL 1051136
#define ASSERT_RESET_BIAS_LATCH 1249282
#define DEASSERT_RESET_BIAS_LATCH 1249283
#define ASSERT_RESET_NEURON_BUF 1249540
#define DEASSERT_RESET_NEURON_BUF 1249541

#define SET_SYN 65536
#define RESET_SYN 0

#define RECURRENT_NONPLASTIC 0 
#define ER_NONPLASTIC 131072
#define W0_NONPLASTIC 262144
#define W1_NONPLASTIC 393216
#define BRD_NONPLASTIC 524288

#define RECURRENT_PLASTIC 0
#define BRD_PLASTIC 131072

#define LOW_STATE_PLASTIC 0
#define HIGH_STATE_PLASTIC 65536

#define NEURONS 256
#define PLASTIC_SYN 256
#define NONPLASTIC_SYN 256

#define DELTA_SPK_TIME_MS 0

//bias param
// CTRL register bit field
#define CTRL_ENABLEIP			0x00000001
#define CTRL_ENABLEINTERRUPT	        0x00000004
#define CTRL_ENABLELOOPBACK		0x00000008
#define CTRL_FLUSHFIFO			0x00000010
#define CTRL_MN256CHIPTYPE		0x00000000

// INterrupt Mask register bit field
#define MSK_TIMEWRAPPING		0x00000080
#define BIAS_PROGRAMMING 		0x80000000
#define DATA_PROGRAMMING		0x00000000
// Coarse Current
#define CC_SSP_SSN  	0x00008000
#define CC_24u  	0x00000000
#define CC_3p2u   	0x00004000
#define CC_0p4u    	0x00002000
#define CC_50n  	0x00006000
#define CC_6p5n 	0x00001000
#define CC_820p		0x00005000
#define CC_105p		0x00003000
#define CC_15p		0x00007000
// I Coarse
#define IC_Id2  	0x00000800
#define IC_Id4   	0x00000400
#define IC_Id8    	0x00000200
#define IC_Id16  	0x00000100
#define IC_Id32 	0x00000080
#define IC_Id64 	0x00000040
#define IC_Id128	0x00000020
#define IC_Id256	0x00000010
// Type of bias
#define BT_LowBias 	0x00000000
#define BT_HighBias	0x00000008

#define BT_CascodeBias  0x00000000
#define BT_NormalBias   0x00000004

#define BT_PBias        0x00000000
#define BT_NBias        0x00000002

#define BT_BiasDisable  0x00000000
#define BT_BiasEnable   0x00000001

///


#define START_DELAY 0 //in ms

typedef struct spike
{
  unsigned int dTclock;
  unsigned int command;
} spike_t;

typedef struct neuelab_gen_reg {
	unsigned int offset;
	char rw;
	unsigned int data;
} neuelab_gen_reg_t;

typedef struct bias_st {
	unsigned int time;
	unsigned int bias;
} bias_t;

void write_generic_neuelab_reg (int fp, unsigned int offset, unsigned int data);

unsigned int read_generic_neuelab_reg (int fp, unsigned int offset);

void usage();

int resetMN256(int handle);

int initBias256(int handle);

int initMN256(int classes,int neuronPerClass,int Input,int handle);

int trainMN256(matrix2Di trial, int label, int binSize,int classes,int handle);

int configureMN256(matrix2Di W,matrix2Di EI,int classes,int neuronPerClass,int Input,int handle);

int runMN256(matrix2Di trial,int binSize, int handle);

int setNonPlastic(int syn,int neur, int sr,int latch,int handle);

int setPlasticLatch(int syn,int neur,int sr,int latch,int handle);

int setPlasticHL(int syn, int neur, int hl,int handle);

int sendSpike(int syn, int neur,double dTimeInMS,int handle);

int sendSpikeBR(int syn,double dTimeInMS,int handle);

int sendGenericCMD(unsigned int command,double dTimeInMS,int handle);

unsigned int ms2clock(double ms);

double clock2s(unsigned int clock,int handle);

void sendStimToNM256(matrix2Di trial, int handle);

void importBynaryBias(FILE *biasList, int fp);

//void readData(long msecRecTime, std::vector< int >* output,int handle);

//void readData(int iter,int handle);

///////////missmatch functions//////////////
/*
void divideDataset(dataset* Dataset,PyObject *pValue,int hiddenNeurons,double train_fraction=0.8,double thr=0,double gain=1);

void trainNetwork(PyObject *pValue,PyObject *NN,int neuronsPerClass,int n_epochs=10);

void read(PyObject *pValue);

void create(int input,PyObject *pValue);*/
