# -*- coding: utf-8 -*-
# @author Mattia Rigotti
# @date Fri Jun 13 16:18:51 EDT 2014
'''
missmatch

Toplevel functions
------------------
- seek_data: looks for datasets in the argument directory or in ./data
- load_data: loads a dataset
- describe_data: loads a dataset and parses samples number, classes, etc.
- sample_train_test: divides a given dataset into a training and a test set

Modules
-------

Example
-------

::

    import missmatch as mm

    dataset = mm.seek_data()  # looks at what datasets are present in ./data
    inputs, labels = mm.load_data(dataset[0]) # load the first dataset

'''

try:
    from .data_management import seek_data, load_data, describe_data, sample_train_test
    from .linear_classifiers import Perceptron, SoftBounds, BinaryStoch, BinaryStochMismatch
    from .ensemble import Ensemble
except ImportError as e:
    import sys
    print >>sys.stderr, '''\
Could not import submodules (exact error was: %s).

There are many reasons for this error the most common one is that you have
either not built the packages or have built (using `python setup.py build`) or
installed them (using `python setup.py install`) and then proceeded to test
missmatch **without changing the current directory**.

Try installing and then changing to another directory before importing
missmatch.
''' % e

__all__ = [
    'seek_data', 'load_data', 'describe_data', 'sample_train_test',
    'Perceptron', 'SoftBounds', 'BinaryStoch', 'BinaryStochMismatch', 'Ensemble'
]
